==> pawm-2.2.2.tar.bz2: pawm-2.2.2/src/keyboard.c <==
#include <stdbool.h>
#include "chain.h"
#include "keyboard.h"
#include "pawindow.h"

struct _binding {
	char *name;
	KeySym keycode;
	long modifier;
	callback action;
	void *param;
};

typedef struct _binding *Binding;

static Chain pawm_bindings;

static void keyboard_add_binding(char *, long, callback, void *);
static void spawn_proccess(void *);


void keyboard_init(void) 
{
	/* Create a chain to store pawm keybindings information*/
    pawm_bindings=chain_create();
    
	/* Add some data. TODO: These information will be retrieved from cfgpool */ 
	
	keyboard_add_binding("e", ControlMask, spawn_proccess, "thinglaunch");
	keyboard_add_binding("F3", Mod1Mask, iconify_client, NULL);
	keyboard_add_binding("F2", Mod1Mask, maximize_client, NULL);
	keyboard_add_binding("F4", Mod1Mask, kill_client, NULL);
	keyboard_add_binding("Tab", Mod1Mask, rotate_clients, "forward");
	keyboard_add_binding("Tab", Mod1Mask|ShiftMask, rotate_clients, "backward");
	keyboard_add_binding("Alt_L", AnyModifier, NULL, NULL);
}


static void keyboard_add_binding(char *name, long modifier, callback action, void *param)
{
	Binding b;
    KeySym ks; KeyCode kc;
	
    ks=XStringToKeysym(name);
    kc=XKeysymToKeycode(display, ks);
	
    b=malloc(sizeof(struct _binding));
	b->name=strdup(name);
	b->keycode=kc;
	b->modifier=modifier;
    b->action=action;
    b->param=param;
	
    chain_put(pawm_bindings, b); 
}


bool keyboard_compare_key (char *name, long modifier, XKeyPressedEvent *event) 
{
    KeySym ks; KeyCode kc;
	
	if (!name) {
		kc=event->keycode;
	} else {
		ks=XStringToKeysym(name);
		kc=XKeysymToKeycode(display, ks);
	}

	if ( event->keycode==kc && ( modifier==AnyModifier || ((event->state & modifier)==modifier)) )
		return true;
	else
		return false;
}


/* Grab all the configured keys for a given window */
void keyboard_grabkeys(Window window)
{
	Binding bdng;

	chain_home(pawm_bindings);

	while ( (bdng=chain_next(pawm_bindings)) != NULL) {
		XGrabKey(display, bdng->keycode, bdng->modifier, window, True, GrabModeAsync, GrabModeAsync);
	}
}


void keyboard_ungrabkeys(Window window)
{
	Binding bdng;

	chain_home(pawm_bindings);

	while ( (bdng=chain_next(pawm_bindings)) != NULL) {
		XUngrabKey(display, bdng->keycode, bdng->modifier, window);
	}
}
	


void keyboard_gotkey(KeyCode keycode, long modifier)
{
	Binding elem;
	int i;
	
	/* Ignore the NumLock modifier */
	if (modifier!=AnyModifier)
		modifier&= ~Mod2Mask;

    chain_home(pawm_bindings);
    for (i=0; i<chain_size(pawm_bindings); i++) {
		elem=chain_next(pawm_bindings);
        if ( (keycode==elem->keycode) && ((modifier==AnyModifier) || ((elem->modifier & modifier)==modifier)) ) {
		    if (elem->action) {
				if (!elem->param) {	
					/* Handlers must expect null values and handle them */
					elem->action(focuspawindow);
				} else {
					elem->action(elem->param);
				}
			}

			break;
		}
	}
}


bool keyboard_binding_key(XKeyPressedEvent *event)
{
	Binding elem;
	int i;
	
    chain_home(pawm_bindings);
    for (i=0; i<chain_size(pawm_bindings); i++) {
		elem=chain_next(pawm_bindings);
        if ( (event->keycode==elem->keycode) && ((event->state==AnyModifier) || ((elem->modifier & event->state)==event->state)) ) {
			return true;
		}
	}

	return false;
}


void keyboard_done(void)
{
	Binding elem;

    chain_seek(pawm_bindings, chain_size(pawm_bindings));
    while ( (elem=chain_del(pawm_bindings))!=NULL ) {
		free(elem);
    }
}


static void spawn_proccess(void *param)
{
	int pid;
	char *cmd;
	
	ASSERT(param);
	
	cmd=(char *)param;

	pid=fork();
	
	if (pid==0) {
		/* Child */
		execl("/bin/sh", "/bin/sh", "-c", cmd, NULL);
	}
}

==> pawm-2.2.7.tar.bz2: pawm-2.2.7/src/keyboard.c <==
#include <stdbool.h>
#include "chain.h"
#include "keyboard.h"
#include "pawindow.h"

struct _binding {
	char *name;
	KeySym keycode;
	long modifier;
	callback action;
	void *param;
};

typedef struct _binding *Binding;

static Chain pawm_bindings;

static void keyboard_add_binding(char *, long, callback, void *);
static void spawn_proccess(void *);


void keyboard_init(void) 
{
	/* Create a chain to store pawm keybindings information*/
    pawm_bindings=chain_create();
    
	/* Add some data. TODO: These information will be retrieved from cfgpool */ 
	
	keyboard_add_binding("e", ControlMask, spawn_proccess, "thinglaunch");
	keyboard_add_binding("F3", Mod1Mask, iconify_client, NULL);
	keyboard_add_binding("F2", Mod1Mask, maximize_client, NULL);
	keyboard_add_binding("F4", Mod1Mask, kill_client, NULL);
	keyboard_add_binding("Tab", Mod1Mask, rotate_clients, "forward");
	keyboard_add_binding("Tab", Mod1Mask|ShiftMask, rotate_clients, "backward");
	keyboard_add_binding("Alt_L", AnyModifier, NULL, NULL);
}


static void keyboard_add_binding(char *name, long modifier, callback action, void *param)
{
	Binding b;
    KeySym ks; KeyCode kc;
	
    ks=XStringToKeysym(name);
    kc=XKeysymToKeycode(display, ks);
	
    b=malloc(sizeof(struct _binding));
	b->name=strdup(name);
	b->keycode=kc;
	b->modifier=modifier;
    b->action=action;
    b->param=param;
	
    chain_put(pawm_bindings, b); 
}


bool keyboard_compare_key (char *name, long modifier, XKeyPressedEvent *event) 
{
    KeySym ks; KeyCode kc;
	
	if (!name) {
		kc=event->keycode;
	} else {
		ks=XStringToKeysym(name);
		kc=XKeysymToKeycode(display, ks);
	}

	if ( event->keycode==kc && ( modifier==AnyModifier || ((event->state & modifier)==modifier)) )
		return true;
	else
		return false;
}


/* Grab all the configured keys for a given window */
void keyboard_grabkeys(Window window)
{
	Binding bdng;

	chain_home(pawm_bindings);

	while ( (bdng=chain_next(pawm_bindings)) != NULL) {
		XGrabKey(display, bdng->keycode, bdng->modifier, window, True, GrabModeAsync, GrabModeAsync);
	}
}


void keyboard_ungrabkeys(Window window)
{
	Binding bdng;

	chain_home(pawm_bindings);

	while ( (bdng=chain_next(pawm_bindings)) != NULL) {
		XUngrabKey(display, bdng->keycode, bdng->modifier, window);
	}
}
	


void keyboard_gotkey(KeyCode keycode, long modifier)
{
	Binding elem;
	int i;
	
	/* Ignore the NumLock/CapsLock modifiers */
	if (modifier!=AnyModifier) {
		modifier&= ~Mod2Mask;
		modifier&= ~LockMask;
	}

    chain_home(pawm_bindings);
    for (i=0; i<chain_size(pawm_bindings); i++) {
		elem=chain_next(pawm_bindings);
        if ( (keycode==elem->keycode) && ((modifier==AnyModifier) || ((elem->modifier & modifier)==modifier)) ) {
		    if (elem->action) {
				if (!elem->param) {	
					/* Handlers must expect null values and handle them */
					elem->action(focuspawindow);
				} else {
					elem->action(elem->param);
				}
			}

			break;
		}
	}
}


bool keyboard_binding_key(XKeyPressedEvent *event)
{
	Binding elem;
	int i;
	
    chain_home(pawm_bindings);
    for (i=0; i<chain_size(pawm_bindings); i++) {
		elem=chain_next(pawm_bindings);
        if ( (event->keycode==elem->keycode) && ((event->state==AnyModifier) || ((elem->modifier & event->state)==event->state)) ) {
			return true;
		}
	}

	return false;
}


void keyboard_done(void)
{
	Binding elem;

    chain_seek(pawm_bindings, chain_size(pawm_bindings));
    while ( (elem=chain_del(pawm_bindings))!=NULL ) {
		free(elem);
    }
}


static void spawn_proccess(void *param)
{
	int pid;
	char *cmd;
	
	ASSERT(param);
	
	cmd=(char *)param;

	pid=fork();
	
	if (pid==0) {
		/* Child */
		execl("/bin/sh", "/bin/sh", "-c", cmd, NULL);
	}
}
